/*
Copyright (C) 2004-2005 Amour Hassan
                                                                                
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
                                                                                
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
                                                                                
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                                
*/

#ifndef _RG3DROM_FILE_H
#define _RG3DROM_FILE_H

#include <fstream>

// Currently, 4MB of memory is the maximum custom tag storage allowed.
// TODO: Add a feature to allow for larger custom tag data sizes to be saved
//	to a large backing store (like a harddrive). This would be usefull,
//	however placing a 4MB limit is safe and it removes the complexity
//	of having to deal with file input/output.
#define SK3DROM_MAXIMUM_CUSTOM_TAG_MEMORY_STORAGE	4194304

#define sk3DROM_HEADER		"sk3DROM "	// File header
#define sk3DROM_VERTEX		0x1001		// Vertices
#define sk3DROM_FACE		0x1002		// Faces
#define sk3DROM_TEXTURE		0x1003		// Textures
#define sk3DROM_TSTRIP		0x1004		// Triangle strips
#define sk3DROM_TEXINFO		0x1005		// Texture information
#define sk3DROM_SKELETON	0x1006		// Skeleton information
#define sk3DROM_SKELFRAMES	0x1007		// Skeleton key frames
#define sk3DROM_VERTMODNAMES	0x1008		// Name for each vertex model

#define sk3DROM_MAIN_SKELETON	0x2000		// Main skeleton
#define sk3DROM_SKELFRAMES_2	0x2001		// New style skel key frames
#define sk3DROM_SFRAME		0x2002		// A single skeleton frame
#define sk3DROM_BONE		0x2010		// Single bone
#define sk3DROM_BONE_2		0x2011		// Single bone
#define sk3DROM_BONE_FRAME	0x2020		// Single bone frame (old)
#define sk3DROM_BONE_FRAME_2	0x2021		// Single bone frame
#define sk3DROM_SKEL_ANIMATION	0x2100		// Skeleton animations
#define sk3DROM_ANI_SEQ		0x2110		// Skeleton sequence
#define sk3DROM_ANI_FRAME	0x2120		// Skeleton sequence frame
#define sk3DROM_ANI_FRAME_2	0x2130		
#define sk3DROM_MODEL_TEXTURES	0x2200		// Encapsulates all textures
#define sk3DROM_TEXTURE_2	0x2220		// A single texture
#define sk3DROM_TEXTURE_3	0x2221		// A single texture
#define sk3DROM_FACE_2		0x2300
#define sk3DROM_VERTEX_2	0x2400
#define sk3DROM_VERTEX_3	0x2401
#define sk3DROM_GROUPS_DATA	0x2500
#define sk3DROM_GROUP		0x2501
#define sk3DROM_PAGES_DATA	0x2600
#define sk3DROM_PAGE		0x2650
#define sk3DROM_CSG_DATA	0x2700
#define sk3DROM_CSG_BRUSH	0x2701

#define sk3DROM_DATA		0xFFFE		// This is a custom data tag.
						// The contents of this tag
						// can be anything (or
						// nothing). sk3DROM_DATA tags
						// MUST appear within other
						// tags and can never be top
						// level tags (since their data
						// is undefined). Top level
						// tags can be used to define
						// data contained within
						// sk3DROM_DATA tags.

#define sk3DROM_CUSTOM		0xFFFF		// This is a custom tag. The
						// first 64 bytes within this
						// tag are the tag's
						// identifier. Additional bytes
						// can be anything.

/* Mode A
 * - Texture Data
 * - Face Data
 * - Vertices
 */
#define sk3DROM_MODEA		0x1

/* Mode B
 * - Texture Data
 * - Face Data
 * - Vertex Data for all of the frames
 */
#define sk3DROM_MODEB		0x2

#define sk3DROM_MODEC		0x3

/* Mode D
 * - All tags contained within a "MODED" tag can be in any arbitrary order.
 */
#define sk3DROM_MODED		0x4

// File Size: 6 bytes (no padding)
#define sk3DROMTAG_FSIZE	6
class sk3DRomTag
{
public:
	unsigned short mode;	// Identifier (2 bytes)
	int size;	// Size of chunk excluding header (4 bytes).
			// That means a tag with no data will have size == 0.
	sk3DRomTag(){}
	sk3DRomTag( const unsigned short m, const int s) { mode = m; size = s;}

	// XXX: Some ostream dervied classes do not support seeking ahead.
	//	Therefore, "write-ahead" must be used. save_space uses
	//	"write-ahead" to simulate a "seek-ahead".
	void save_space( std::ostream &des) const {
		char tmp[sk3DROMTAG_FSIZE];
		des.write( tmp, sk3DROMTAG_FSIZE);
	}
};

// File Size: 14 bytes (no padding)
#define sk3DROMHEADER_FSIZE	14
struct sk3DRomHeader
{
	char header[8];
	sk3DRomTag ftype;
};

// Description: If the identifier id matches the identifier read from the file,
//	then 1 is returned. If the identifier id does not match the
//	identifier read from the file, then 0 is returned. On error, a negative
//	value is returned. It is an error if id is a NULL pointer.
int sk3DRom_read_and_compare_custom_identifier( std::ifstream &inf,
        const char *id);

#endif
