/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "rgTime.h"

#ifdef WINDOWS_OS
// For Windows, the function queryPerformanceCounter will be used for timing.
// Unix-Like operating systems provide "gettimeofday" for timing.
#include <windows.h>

bool performance_counter_ready = false;
#else
#include <sys/time.h>
#endif

rgTimer::rgTimer()
{
	init();
        time_unit = 1.0;    // By default a time unit is one second
	reset();
}

rgTimer::rgTimer( const double sec)
{
	init();
        time_unit = sec;
	reset();
}

void rgTimer::init()
{
#ifdef WINDOWS_OS
	// The performance counter must be initialized before it is used.
	if( !performance_counter_ready) {
		LARGE_INTEGER temp;
		QueryPerformanceCounter( &temp);
		performance_counter_ready = true;
	}
#endif
}

double rgTimer::elapsed()
{
#ifdef WINDOWS_OS
        // FIXME: The system might not have a high performance counter.
        LARGE_INTEGER count;
        LARGE_INTEGER hz;
        QueryPerformanceFrequency( &hz);
        QueryPerformanceCounter( &count);

        double cur_sec = double( count.QuadPart) / double( hz.QuadPart);
#else
        timeval t;
	gettimeofday( &t, 0);
	double cur_sec = double( t.tv_sec) + ((1.e-6) * double(t.tv_usec));
#endif

	double sec_elapsed = cur_sec - sec_prev;
	sec_prev = cur_sec;

	return sec_elapsed;
}

bool rgTimer::time_unit_passed() const
{
#ifdef WINDOWS_OS
        // FIXME: The system might not have a high performance counter.
        LARGE_INTEGER count;
        LARGE_INTEGER hz;
        QueryPerformanceFrequency( &hz);
        QueryPerformanceCounter( &count);

        double cur_sec = double( count.QuadPart) / double( hz.QuadPart);
#else
	timeval t;
	gettimeofday( &t, 0);

	double cur_sec = double( t.tv_sec) + ((1.e-6) * double(t.tv_usec));
#endif

	double sec_elapsed = cur_sec - sec_prev;
	if( sec_elapsed >= time_unit) return true;
	return false;
}

void rgTimer::reset()
{
#ifdef WINDOWS_OS
        // FIXME: The system might not have a high performance counter.
        LARGE_INTEGER count;
        LARGE_INTEGER hz;
        QueryPerformanceFrequency( &hz);
        QueryPerformanceCounter( &count);
        sec_prev = double( count.QuadPart) / double( hz.QuadPart);
#else
	timeval t;
	gettimeofday( &t, 0);
	sec_prev = double( t.tv_sec) + ((1.e-6) * double(t.tv_usec));
#endif
}

rgFPSCounter::rgFPSCounter()
{
	timer.set_time_unit( 1.0);
	fps = 0.0;
        count_reset = 0;
}

rgFPSCounter::rgFPSCounter( double sec)
{
	timer.set_time_unit( sec);
	fps = 0.0;
        count_reset = 0;
}

void rgFPSCounter::inc_frame()
{
	if( timer.time_unit_passed()) {
		fps = fcount / timer.elapsed();
		fcount = 0.0;
                count_reset++;
	}

	fcount++;
}

void rgFPSCounter::set_time_unit( double sec)
{
	timer.set_time_unit( sec);
}

bool rgFPSCounter::time_unit_passed()
{
	return timer.time_unit_passed();
}


double rgTime_delay( const double sec)
{
	rgTimer t( sec);
	while( !(t.time_unit_passed()));
	return t.elapsed_scalar();
}
